pub struct EliminateJoin;Expand description
Rewrites an inner join to a semi join when one input only filters the other, removes an outer join whose non-preserved side is unused and cannot multiply the preserved side’s rows, and replaces an always-false inner join with an empty relation.
Implementations
Sourceimpl EliminateJoin
impl EliminateJoin