Optimizer rule to replace LIMIT 0 or LIMIT whose ancestor LIMIT's skip is greater than or equal to current's fetch on a plan with an empty relation. This saves time in planning and executing the query.

Structs

Optimization rule that replaces LIMIT 0 or LIMIT whose ancestor LIMIT’s skip is greater than or equal to current’s fetch with an LogicalPlan::EmptyRelation