Function datafusion_optimizer::utils::split_binary
source · Expand description
Splits an binary operator tree Expr such as A <OP> B <OP> C => [A, B, C]
See split_binary_owned for more details and an example.
Splits an binary operator tree Expr such as A <OP> B <OP> C => [A, B, C]
See split_binary_owned for more details and an example.